-- This set of two functions allow use of commas easier in Euphoria.
-- This code is placed in the public domain. Use it for whatever purpose
-- you like for free and at your own risk (it's just Euphoria code, nothing
-- else, so if it messes something up, it's not my fault! :).
-- All I ask is a little credit wherever the credits go. (You can get away
-- from doing this if you don't have a credits area... ;)

-- 1998 Robert Brandon Pilkington (bpilkington@juno.com)


-----------
-- Usage --
-----------

-- s1 = add_commas(a, s2)
--
-- Returns atom a using printf/sprintf style fields for s2 in sequence s1.
-- Example:
--
-- puts(1, add_commas(-1234.5, ".2")
--
-- displays:
-- 
-- -1,234.50
--
-- Field widths are important. It counts the ENTIRE number including commas
-- negative sign, and decimal points. If you don't send at least ".0" you'll
-- get a decimal number in return. (the routine gets the sequence using a
-- floating point call to sprintf())
-- If you want to add commas to strings, then you can call:
--
-- add_commas(remove_commas(s), ".0")
--
-- Where s is the string and ".0" is the field widths you want.
--
-- If you try
--
-- add_commas(1000, "010.0")
-- 
-- You will get some strange results, so don't do it.
--
-- Bug fix: Now supports calls like add_commas(1000, "-5") for left
--          justification.
--
-- See also: remove_commas()

-- a = remove_commas(s)
--
-- Removes commas, dollar signs and percent signs from sequence s and returns
-- in atom a. a can be an integer as long as the number returned is an
-- integer. Very useful for getting string input allowing the user to use
-- commas and other stuff for a professional touch (and removing them to use
-- the number behind the user's back and adding them back with add_commas()
-- when displaying the data :)
--
-- Example:
--
-- printf(1, "%.2f", {remove_commas("$4,400,000.43")})
--
-- displays:
--
-- 4400000.43

include get.e

global function add_commas(atom number, sequence field)
    -- Adds commas to numbers to make them more readable.
    sequence string, f_ws, b_ws
    integer point

    -- Setup the string
    string = sprintf("%" & field & "f", {number})

    -- Find the last whitespace, "   5". A field of "05.0" isn't handled,
    -- because it would be stupid to put commas in like that....
    -- (What would be right, "00,005,000", or "00005,000"???)
    f_ws = ""   -- Front whitespace
    while string[1]=' ' or string[1]='-' or string[1]='+' do
        f_ws = append(f_ws, string[1])
        string = string[2..length(string)]
    end while

    -- Check for whitespace at end of number (ie field = "-5.")
    b_ws = ""   -- Back whitespace
    while string[length(string)] = ' ' do
        b_ws = append(b_ws, string[length(string)])
        string = string[1..length(string)-1]
    end while

    -- Find decimal point, if it exists
    point = find('.', string)
    if point = 0 then
        point = length(string)+1
    end if

    for i = point-4 to 1 by -3 do
        string = string[1..i] & ',' & string[i+1..length(string)]
        if length(f_ws) > 0 then            -- Remove extra whitespace
            if f_ws[1] = ' ' then           -- from the front, but leave the
                f_ws = f_ws[2..length(f_ws)]-- + and - signs.
            end if
        end if
        if length(b_ws) > 0 then            -- Remove extra whitespace
            b_ws = b_ws[1..length(b_ws)-1]  -- from the rear.
        end if
    end for
    return(f_ws & string & b_ws)    -- Restore remaining whitespace
end function

global function remove_commas(sequence num)
    -- Turn the sequence with commas and dollar signs into a usable atom
    object result
    integer c
    c = 0
    while c < length(num) do
        c = c + 1
        if find(num[c], "$,%") then
            num = num[1..c-1] & num[c+1..length(num)]
            c = c - 1
        end if
    end while
    result = value(num)
    if result[1] = GET_SUCCESS then
        result = result[2]
    else
        result = 0
    end if
    return result
end function

